using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Reflection;
using CSScriptLibrary;

namespace CSScriptIntellisense
{
    public static class CSSctiptHelper
    {
        static public Tuple<string[], string[]> GetProjectFiles(string script)
        {
            var searchDirs = new List<string> { Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location), ScriptsDir, };

            var nppScriptsAsm = AppDomain.CurrentDomain.GetAssemblies()
                                         .Where(x => x.FullName.StartsWith("NppScripts,"))
                                         .Select(x=>x.Location)
                                         .FirstOrDefault();

            var parser = new ScriptParser(script, searchDirs.ToArray(), false);
            searchDirs.AddRange(parser.SearchDirs);        //search dirs could be also defined n the script

            IList<string> sourceFiles = parser.SaveImportedScripts().ToList(); //this will also generate auto-scripts and save them
            sourceFiles.Add(script);

            //some assemblies are referenced from code and some will need to be resolved from the namespaces
            var refAsms = parser.ReferencedNamespaces
                                .Where(name => !parser.IgnoreNamespaces.Contains(name))
                                .SelectMany(name => AssemblyResolver.FindAssembly(name, searchDirs.ToArray()))
                                .Union(parser.ReferencedAssemblies
                                             .SelectMany(asm => AssemblyResolver.FindAssembly(asm, searchDirs.ToArray())))
                                .Distinct()
                                .ToList();

            if (nppScriptsAsm != null)
                refAsms.Add(nppScriptsAsm);

            return new Tuple<string[], string[]>(sourceFiles.ToArray(), refAsms.ToArray());
        }

        static public string ScriptsDir
        {
            get
            {
                string rootDir = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);
                string scriptDir = Path.Combine(rootDir, "NppScripts");

                if (!Directory.Exists(scriptDir))
                    Directory.CreateDirectory(scriptDir);

                return scriptDir;
            }
        }
    }
}
