using System;
using System.ComponentModel;
using System.IO;
using System.Reflection;
using System.Runtime.InteropServices;
using System.Text;

namespace CSScriptIntellisense
{
    /// <summary>
    /// Of course XML based config is more natural, however ini file reading (just a few values) 
    /// is faster. 
    /// </summary>
    public class Config : IniFile
    {
        static Config instance;
        
        public static Config Instance
        {
            get
            {
                if(instance == null)
                    instance = new Config();
                return instance;
            }
        }

        public Config(string file)
            : base(file)
        {
            Open();
        }

        public Config()
        {
            var configDir = Path.Combine(Npp.GetConfigDir(), "CSharpIntellisense");
            
            if(!Directory.Exists(configDir))
                Directory.CreateDirectory(configDir);
            string path = Path.Combine(configDir, "settings.ini");
            
            base.file = path;
            Open();
        }

        public bool UseArrowToAccept = true;
        public bool InterceptCtrlSpace = true;
        public bool NewPluginConfictReported = false;
        public bool ShowQuickInfoInStatusBar = false;
        public bool UseMethodBrackets = false;

        public void Save()
        {
            SetValue("settings", "NewPluginConfictReported", NewPluginConfictReported);
            SetValue("settings", "UseArrowToAccept", UseArrowToAccept);
            SetValue("settings", "InterceptCtrlSpace.v2", InterceptCtrlSpace);
            SetValue("settings", "UseMethodBrackets", UseMethodBrackets);
            SetValue("settings", "ShowQuickInfoInStatusBar", ShowQuickInfoInStatusBar);
        }

        public void Open()
        {
            NewPluginConfictReported = GetValue("settings", "NewPluginConfictReported", NewPluginConfictReported);
            UseArrowToAccept = GetValue("settings", "UseArrowToAccept", UseArrowToAccept);
            InterceptCtrlSpace = GetValue("settings", "InterceptCtrlSpace.v2", InterceptCtrlSpace);
            UseMethodBrackets = GetValue("settings", "UseMethodBrackets", UseMethodBrackets);
            ShowQuickInfoInStatusBar = GetValue("settings", "ShowQuickInfoInStatusBar", ShowQuickInfoInStatusBar);
        }
    }
}
