using System;
using System.Linq;
using Xunit;

namespace CSScriptIntellisense.Test
{
    public class MethodInfoTest
    {
        [Fact]
        public void CountArgumentsInMemberInfo()
        {
            Assert.Equal(0, NRefactoryExtensions.GetArgumentCount("Method: void Console.WriteLine()"));
            Assert.Equal(1, NRefactoryExtensions.GetArgumentCount("Method: void Console.WriteLine(bool value)"));
            Assert.Equal(2, NRefactoryExtensions.GetArgumentCount("Method: void Console.WriteLine(bool value, object arg0)"));
            Assert.Equal(2, NRefactoryExtensions.GetArgumentCount("Method: void Class.Method(string text, Dictionary<int, string> map)"));
            Assert.Equal(3, NRefactoryExtensions.GetArgumentCount("Method: void Class.Method(string text, Dictionary<int, string> map, int count)"));
        }

        [Fact]
        public void ProcessMethodOverloadsHint()
        {
            SimpleCodeCompletion.ResetProject();

            //Simulate invoking ShowMathodInfo
            //Console.WriteLine(|
            string[] signatures = SimpleCodeCompletion.GetMemberInfo(@"using System;
using System.Linq;

class Script
{
    static public void Main(string[] args)
    {
        Console.WriteLine(args.Length);
    }
}", 131, "test.cs", false);

            Assert.Equal(19, signatures.Count()); // may need to be updated for the new .NET versions

            //Simulate typing...
            //Console.WriteLine("Time {0}", DateTime.|

            var popup = new MemberInfoPanel();

            popup.AddData(signatures);
            Assert.Equal(19, popup.items.Count);

            popup.ProcessMethodOverloadHint(new[]{"Time {0}"});  //'single and more' parameter methods
            Assert.Equal(18, popup.items.Count);  

            popup.ProcessMethodOverloadHint(new[] { "\"Time {0}\"", "DateTime." });  //'two and more' parameter methods
            Assert.Equal(6, popup.items.Count);  

        }
    }
}